package com.mediaworx.b4f.opencms.contenttypes.tags;

import com.mediaworx.b4f.opencms.contenttypes.ContentTypeConfiguration;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.json.simple.JSONValue;
import org.opencms.file.CmsObject;
import org.opencms.file.CmsPropertyDefinition;
import org.opencms.file.CmsResource;
import org.opencms.file.CmsResourceFilter;
import org.opencms.flex.CmsFlexController;
import org.opencms.main.CmsException;

import javax.servlet.ServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 *  Prints out a JSON String of list folders or list entities in a specific folder (if the request parameter
 *  <code>listname</code> is set). Lists are located in the OpenCms VFS under <code>/.content/listentries</code>.
 *  Used to connect the {@link com.mediaworx.b4f.opencms.contenttypes.widgets.ListSelectorWidget} to the backend
 *  to dynamically retrieve list names and list entries via AJAX.
 */
public class ListSelectorOptionsTagHandler extends TagSupport {

	private static final long serialVersionUID = 3370736324897030129L;
	private static final Log LOG = LogFactory.getLog(ListSelectorOptionsTagHandler.class);

	/** key for the list folder's/entrie's URI, used in the JSON string */
	private static final String KEY_URI = "uri";

	/** key for the list folder's/entrie's name, used in the JSON string */
	private static final String KEY_NAME = "name";

	/**
	 * <p>
	 * If the request parameter <code>listname</code> is NOT set:<br/>
	 * reads the list folders under <code>/.content/listentries</code> and prints out a JSON String of the folder's
	 * titles (as "name") and filenames (as "uri").
	 * </p><p>
	 * Example:<br />
	 * <code>
	 *     [{"name":"FAQ","uri":"FAQ"},{"name":"Garantie","uri":"Garantie"},{"name":"Information","uri":"Information"},{"name":"Ringschablone","uri":"Ringschablone"},{"name":"Waschsymbole","uri":"Waschsymbole"}]
	 * </code>
	 * </p><p>
	 * If the request parameter <code>listname</code> IS set:<br/>
	 * reads the list entries under <code>/.content/listentries/[listname]</code> and prints out a JSON String of the
	 * entry's titles (as "name") and URIs (as "uri").
	 * </p><p>
	 * Example:
	 * <code>
	 *     [{"name":"Garantie Karibu","uri":"\/.content\/listentries\/Garantie\/Garantie_Karibu.html"},{"name":"Garantie Reflexion Zertifikat Galia","uri":"\/.content\/listentries\/Garantie\/Garantie_Reflexion_Zertifikat_Galia.html"},{"name":"Garantie XEN","uri":"\/.content\/listentries\/Garantie\/Garantie_XEN.html"}]
	 * </code>
	 * </p>
	 *
	 * @return  always returns SKIP_BODY
	 * @throws JspException     the JspException is never thrown
	 */
	@Override
	public int doStartTag() throws JspException {

		LOG.trace("ListSelectorOptionsTagHandler.doStartTag called");

		ServletRequest request = pageContext.getRequest();
		String listName = request.getParameter("listname");

		boolean getListNames = listName == null || listName.length() == 0;
		String uri;
		String basePath = ContentTypeConfiguration.getListEntriesVfsBasePath();

		if (getListNames) {
			uri = basePath;
		}
		else {
			uri = basePath + listName + "/";
		}

		if (LOG.isTraceEnabled()) {
			LOG.trace("uri: "+uri);
		}

		// get the current users OpenCms context
		CmsObject cms = CmsFlexController.getCmsObject(pageContext.getRequest());

		List<Map<String, String>> listEntities = new LinkedList<Map<String, String>>();  // Map: key - URI, value - list name or entry name
		try {
			CmsResourceFilter resourceFilter = getListNames ? CmsResourceFilter.DEFAULT_FOLDERS : CmsResourceFilter.DEFAULT_FILES;
			List<CmsResource> listResources = cms.readResources(uri, resourceFilter, false);
			for (CmsResource listResource : listResources) {
				if (getListNames && listResource.getName().equals(".Trash")) {
					if (LOG.isTraceEnabled()) {
						LOG.trace("It's the trash folder, skipping: " + listResource.getName());
					}
					continue;
				}
				Map<String, String> listEntity = new HashMap<String, String>();
				listEntity.put(KEY_URI, getListNames ? listResource.getName() : cms.getSitePath(listResource));
				String title = cms.readPropertyObject(listResource, CmsPropertyDefinition.PROPERTY_TITLE, false).getValue(listResource.getName());
				listEntity.put(KEY_NAME, title);
				listEntities.add(listEntity);
			}
			try {
				JSONValue.writeJSONString(listEntities, pageContext.getOut());
			}
			catch (IOException e) {
				LOG.error("Exception printing to out", e);
			}
		}
		catch (CmsException e) {
			LOG.error("Exception reading list folders: " + e.getMessage());
		}

		return SKIP_BODY;
	}

}
