package com.mediaworx.b4f.opencms.contenttypes.widgets;

import com.mediaworx.b4f.opencms.contenttypes.ContentTypeConfiguration;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.opencms.file.CmsObject;
import org.opencms.file.CmsResource;
import org.opencms.i18n.CmsMessages;
import org.opencms.main.OpenCms;
import org.opencms.widgets.A_CmsWidget;
import org.opencms.widgets.I_CmsADEWidget;
import org.opencms.widgets.I_CmsWidgetDialog;
import org.opencms.widgets.I_CmsWidgetParameter;
import org.opencms.xml.types.A_CmsXmlContentValue;

import java.util.*;

/**
 * Generic class defining methods and fields used by all custom widgets developed for the b4f Product CMS
 */
public abstract class B4fWidget extends A_CmsWidget implements I_CmsADEWidget {

	private static final Log LOG = LogFactory.getLog(B4fWidget.class);

	/** Map storing the widget's configuration (taken from the attribute "configuration" of the respective xsd-file) */
	protected Map<String, String> m_configurationMap;

	/**
	 * module base path for widget resources (js/css), configured as <code>WidgetResources.basePath</code> in
	 * <code>ContentTypeConfig.properties</code>
	 */
	private static final String RESOURCES_PATH = ContentTypeConfiguration.getWidgetResourcesBasePath();
	// konfiguriert auf "/system/modules/com.mediaworx.b4f.opencms.contenttypes/resources/"

	/**
	 * export path for widget resources (js/css), configured as <code>WidgetResources.exportBasePath</code> in
	 * <code>ContentTypeConfig.properties</code>
	 */
	protected final String RESOURCES_EXPORT_PATH = ContentTypeConfiguration.getWidgetResourcesExportBasePath();
	// konfiguriert auf "/b4f/"

	/** boolean flag indicating if the widget has at least one export path for resources */
	protected final boolean HAS_RESOURCES_EXPORT_PATH = RESOURCES_EXPORT_PATH != null && RESOURCES_EXPORT_PATH.length() > 0;

	/** String used to separate distinct configuration entities */
	protected static final String CONFIG_ENTITYSEPARATOR = "&";

	/** String used to assign configuration values to keys */
	protected static final String CONFIG_KEYVALUESEPARATOR = "=";


	/**
	 * Creates a new B4fWidget with the provided configuration
	 *
	 * @param configuration configuration string (from the attribute "configuration" in the xsd file)
	 */
	public B4fWidget(String configuration) {
		super(configuration);
		if (LOG.isTraceEnabled()) {
			LOG.trace("B4fWidget (" + this.getClass().getSimpleName() + ") initialized with configuration: " + configuration);
		}
	}

	/**
	 * initializes the widget configuration with the configuration String provided in the respective XSD file
	 *
	 * @see org.opencms.widgets.I_CmsWidget#setConfiguration(String)
	 */
	@Override
	public void setConfiguration(String configuration) {
		m_configurationMap = new HashMap<>();
		if (configuration != null && configuration.length() > 0) {
			String[] keyValuePairs = configuration.split("\\|");
			for (String keyValuePair : keyValuePairs) {
				String[] keyValue = keyValuePair.split("=");
				try {
					m_configurationMap.put(keyValue[0], keyValue[1]);
				}
				catch (ArrayIndexOutOfBoundsException e) {
					LOG.error("Can't parse configuration String " + keyValuePair, e);
				}
			}
		}
	}

	/**
	 * returns the configuration string that is passed to the JavaScript widget
	 *
	 * @param cms           the current OpenCms object
	 * @param schemaType    the schema type
	 * @param messages      the messages
	 * @param resource      the edited resource
	 * @param contentLocale the content locale
	 * @return configuration string that is passed to the JavaScript widget
	 */
	@Override
	public String getConfiguration(
			CmsObject cms,
			A_CmsXmlContentValue schemaType,
			CmsMessages messages,
			CmsResource resource,
			Locale contentLocale) {

		StringBuilder configuration = new StringBuilder(64);

		Iterator<String> it = m_configurationMap.keySet().iterator();
		for (int i = 0; it.hasNext(); i++) {
			if (i > 0) {
				configuration.append(CONFIG_ENTITYSEPARATOR);
			}
			String key = it.next().toLowerCase();
			configuration.append(key).append(CONFIG_KEYVALUESEPARATOR).append(m_configurationMap.get(key));
		}

		if (configuration.length() > 0) {
			configuration.append(CONFIG_ENTITYSEPARATOR);
		}

		// construct the name attribute for form elements
		// OpenCmsProductDescription/CustomWidgetTest
		String typeName = schemaType.getContentDefinition().getTypeName();
		String nodeName = schemaType.getName();
		String elementName = typeName + ":" + nodeName;
		configuration.append("name").append(CONFIG_KEYVALUESEPARATOR).append(elementName);

		// the returned string will be passed on to the client side widget
		return configuration.toString();
	}

	/**
	 * not needed when using ADE (Advanced Direct Edit, standard in OpenCms 8.5), but mandatory because of the
	 * interface. Not used for b4f widgets.
	 *
	 * @param cms           the current OpenCms object
	 * @param widgetDialog  the dialog containing the widget
	 * @param param         the widget parameter to generate the widget for
	 * @return Always returns <code>null</code>
	 */
	@Override
	public String getDialogWidget(CmsObject cms, I_CmsWidgetDialog widgetDialog, I_CmsWidgetParameter param) {
		return null;
	}

	/**
	 * Since all b4f widgets are external custom widgets, false is returned.
	 *
	 * @return Always returns <code>false</code>
	 */
	@Override
	public boolean isInternal() {
		// this is no internal core widget, so return false
		return false;
	}

	/**
	 * returns the VFS path to widget resources (js, css)
	 * @return String, VFS path to widget resources
	 */
	protected String getResourcesPath() {
		return RESOURCES_PATH;
	}

	/**
	 * returns a list of css paths. These are included in the XML content editor and the inline editor.
	 *
	 * @param cms  the current OpenCms object
	 * @return String-List of css paths.
	 */
	@Override
	public List<String> getCssResourceLinks(CmsObject cms) {
		if (LOG.isTraceEnabled()) {
			LOG.trace("getCssResourceLinks called");
		}
		List<String> cssList = new ArrayList<String>();
		for (String cssPath : getCssPaths()) {
			cssList.add(getResourceLink(cms, cssPath));
		}
		return cssList;
	}

	/**
	 * returns a list of JavaScript paths. These are included in the XML content editor and the inline editor.
	 *
	 * @param cms  the current OpenCms object
	 * @return String-List of JavaScript paths.
	 */
	@Override
	public List<String> getJavaScriptResourceLinks(CmsObject cms) {
		if (LOG.isTraceEnabled()) {
			LOG.trace("getJavaScriptResourceLinks called");
		}
		List<String> jsList = new ArrayList<String>();
		for (String jsPath : getJsPaths()) {
			jsList.add(getResourceLink(cms, jsPath));
		}
		return jsList;
	}

	/**
	 * gets the full resource path for a JavaScript or CSS
	 * @param cms      the current OpenCms object
	 * @param subPath  the partial path (e.g. "ckeditor/ckeditor.js")
	 * @return  the full resource path for a JavaScript or CSS
	 */
	private String getResourceLink(CmsObject cms, String subPath) {
		if (HAS_RESOURCES_EXPORT_PATH) {
			return RESOURCES_EXPORT_PATH + subPath;
		}
		else {
			return OpenCms.getLinkManager().substituteLink(cms, getResourcesPath() + subPath);
		}
	}

	/**
	 * must return paths to css files (relative to RESOURCESPATH) that will be integrated in the xml content editor and
	 * the inline editor
	 * @return String array containing css paths
	 */
	protected abstract String[] getCssPaths();

	/**
	 * must return paths to JavaScript files (relative to RESOURCESPATH) that will be integrated in the xml content
	 * editor and the inline editor
	 * @return String array containing JavaScript paths
	 */
	protected abstract String[] getJsPaths();

}
